#include <sys/stat.h>
#include <stdio.h>

#include "SDL/SDL.h"

#include "xgaview.h"

/* DrawPixel function adapted from:
 * http://www.libsdl.org/intro.en/usingvideo.html
 */

void DrawPixel(SDL_Surface *screen, int x, int y, int colour)
{
  if ( SDL_MUSTLOCK(screen) )
    if ( SDL_LockSurface(screen) < 0 )
      return;
  Uint32 *bufp;
  bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
  *bufp = colour;
  if ( SDL_MUSTLOCK(screen) )
    SDL_UnlockSurface(screen);
  SDL_UpdateRect(screen, x, y, 1, 1);
}

void DrawTile(SDL_Surface * screen, Tile tile, unsigned int tilesize,
    unsigned int x, unsigned int y, int scale_factor)
{
  int i = 0;
  int j = 0;
  int k = 0;

  if ( SDL_MUSTLOCK(screen) )
    if ( SDL_LockSurface(screen) < 0 )
      return;

  for (i = 0; i < tilesize * tilesize; i++)
  {
    int internal_x = x + (i % tilesize) * scale_factor;
    int internal_y = y + (i / tilesize) * scale_factor;
    int colour = tile[i];
    
    for (j = 0; j < scale_factor; j++)
    {
      for (k = 0; k < scale_factor; k++)
      {
        Uint32 * bufp = (Uint32 *)screen->pixels +
            (internal_y + j) * screen->pitch / 4 + (internal_x + k);
        *bufp = colour;
      }
    }
  }

  if ( SDL_MUSTLOCK(screen) )
    SDL_UnlockSurface(screen);
  SDL_UpdateRect(screen, x, y, tilesize * scale_factor, tilesize * scale_factor);
}

int main(int argc, char ** argv)
{
  //window variables
  int quit = 0;
  const unsigned short int window_width = 640;
  const unsigned short int window_height = 480;
  SDL_Event event;
  SDL_Surface * screen = NULL;

  //direction variables
  int left;
  int right;
  int up;
  int down;
  
  //graphics variables
  int x = 0; //coordinates of top-left tile in window
  int y = 0;
  const unsigned short int tilesize = 16;
  const unsigned short int gfxmode = EGA;
  const unsigned short int scale_factor = 2;
  unsigned int offset = 0; //tile offset for tile viewer (obsolete)
  int tileview = 1;

  SDL_Init(SDL_INIT_EVERYTHING);
  screen = SDL_SetVideoMode(window_width, window_height, 32, SDL_SWSURFACE);

  Tileset tileset = getTileset("../U1/T1KTILES.BIN", gfxmode, tilesize);
  Tile tile = getTile(tileset, offset, gfxmode, tilesize);

  Map map = loadMap("../U1/MAP.BIN", 168, 156);

  for (int i = 0; i < window_width / (tilesize * scale_factor); i++)
    for (int j = 0; j < window_height / (tilesize * scale_factor); j++)
      DrawTile(screen, tileset.tiles[map.tiles[(y + j) * map.width + (x + i)]],
          tilesize, i * tilesize * scale_factor,
          j * tilesize * scale_factor, scale_factor);

  while (quit == 0)
  {
    SDL_PollEvent(&event); //formerly used SDL_WaitEvent
    SDL_Delay(30);         //program must not use 100% CPU and must not be too fast
    switch(event.type)
    {
      case SDL_QUIT:
        quit = 1;
        break;
      case SDL_KEYDOWN:
        switch(event.key.keysym.sym) 
        {
          case SDLK_t:
            tileview = !tileview;
            break;
          case SDLK_PAGEUP:
            if (offset > 0)
              offset--;
            break;
          case SDLK_PAGEDOWN:
            if (offset < tileset.count - 1)
              offset++;
            break;
          case SDLK_s:
            {
              SDL_Surface * entiremap = NULL;
              entiremap = SDL_SetVideoMode(map.width * scale_factor * tilesize,
                  map.height * scale_factor * tilesize, 32, SDL_SWSURFACE);
              for (int i = 0; i < map.width; i++)
                for (int j = 0; j < map.height; j++)
                  DrawTile(entiremap, tileset.tiles[map.tiles[j * map.width + i]],
                      tilesize, i * tilesize * scale_factor,
                      j * tilesize * scale_factor, scale_factor);
              SDL_SaveBMP(entiremap, "u1map.bmp");
              SDL_FreeSurface(entiremap);
              screen = SDL_SetVideoMode(window_width, window_height, 32, SDL_SWSURFACE);
            }
          case SDLK_DOWN:
            down = 1;
            break;
          case SDLK_UP:
            up = 1;
            break;
          case SDLK_LEFT:
            left = 1;
            break;
          case SDLK_RIGHT:
            right = 1;
            break;
          case SDLK_ESCAPE:
            quit = 1;
            break;     
        }
        break;
      case SDL_KEYUP:
        switch(event.key.keysym.sym) 
        {
          case SDLK_DOWN:
            down = 0;
            break;
          case SDLK_UP:
            up = 0;
            break;
          case SDLK_LEFT:
            left = 0;
            break;
          case SDLK_RIGHT:
            right = 0;
            break;
        }
        break;
    }
      if (up == 1)
            if (y > 0)
              y--;
      if (down == 1)
            if (y < (map.height - (window_height / (tilesize * scale_factor)) - 1))
              y++;
      if (left == 1)
            if (x > 0)
              x--;
      if (right == 1)
            if (x < (map.width - (window_width / (tilesize * scale_factor)) - 1))
              x++;
      
        //tile = getTile(tileset, offset, gfxmode, tilesize);
        for (int i = 0; i < window_width / (tilesize * scale_factor); i++)
          for (int j = 0; j < window_height / (tilesize * scale_factor); j++)
            DrawTile(screen, tileset.tiles[map.tiles[(y + j) * map.width + (x + i)]],
                tilesize, i * tilesize * scale_factor,
                j * tilesize * scale_factor, scale_factor);
        if (tileview == 1)
        {
          Tile tile = getTile(tileset, offset, gfxmode, tilesize);
          DrawTile(screen, tile, tilesize, 0, 0, scale_factor);
        }
        /*
        for (int i = 0; i < tilesize * tilesize; i++)
          DrawPixel(screen, i % tilesize, i / tilesize, tile[i]);
        */
  }
  //cleaning up
  free(tile);
  free(tileset.tiles);
  free(map.tiles);
  SDL_FreeSurface(screen); //free memory taken up by screen surface
  SDL_Quit(); //close SDL library
  return 0;
}
